// HSSelectionSet.cpp : implementation of the HPSelectionSet class
// 

#include "HSSelectionSet.h"

#include "HSolidModel.h"
#include "HSolidView.h"

#include "HBaseOperator.h"

#include "hc.h"

#ifdef PARASOLID
#include "hp_bridge.h"
#endif

#ifdef ACIS
#include "ha_bridge.h"

// Required to use ENTITY class
#include "kernel/kerndata/data/entity.hxx"

// Required to use constant  FACE_TYPE
#include "kernel/kerndata/top/face.hxx"

// Required to use constant  EDGE_TYPE
#include "kernel/kerndata/top/edge.hxx"

// Required to use constant  BODY_TYPE
#include "kernel/kerndata/top/body.hxx"
#endif

#include <assert.h>



HSSelectionSet::HSSelectionSet(HBaseView* view):HSelectionSet(view)
{
#ifdef PARASOLID
	m_SelectLevel = PK_CLASS_body;	// set default level to body
#endif
#ifdef ACIS
	m_SelectLevel = BODY_TYPE;
#endif

	m_pSolidSelection = 0;
}


HSSelectionSet::~HSSelectionSet()
{
	delete m_pSolidSelection;
}

// create a new list  object
void HSSelectionSet::Init()
{
	m_pSolidSelection = new HVarray();

	HSelectionSet::Init();
}


long HSSelectionSet::GetAtSolidEntity(int index)
{
	return m_pSolidSelection->At(index);
}


int HSSelectionSet::GetSolidListSize()
{
	return m_pSolidSelection->Size();
}


void HSSelectionSet::Select(long key, char* segpath, long include_key, long includer_key)
{
	HC_KEY	keys[40000];
	int		count = 0, c;

#ifdef PARASOLID
		PK_ENTITY_t	entity = 0;

		entity = HP_Compute_TagID(key, m_SelectLevel);

		if (entity > 0)
		{
			m_pSolidSelection->Add((long)entity);

			if (m_SelectLevel == PK_CLASS_body)  // && body segments is true 
				count = HP_Compute_Geometry_Keys(entity, 1, keys, "bodies");
			else if (m_SelectLevel == PK_CLASS_face)
				count = HP_Compute_Geometry_Keys(entity, 40000, keys, "faces");
			else if (m_SelectLevel == PK_CLASS_edge)
				count = HP_Compute_Geometry_Keys(entity, 40000, keys, "edges");
		}
#endif
#ifdef ACIS
		ENTITY* entity = 0;

		entity = HA_Compute_Entity_Pointer(key, m_SelectLevel);

		if (entity)
		{
			m_pSolidSelection->Add((long)entity);

			if (m_SelectLevel == BODY_TYPE)  // && 'body segments is true 
				count = HA_Compute_Geometry_Keys(entity, 1, keys, "bodies");
			else if (m_SelectLevel == FACE_TYPE)
				count = HA_Compute_Geometry_Keys(entity, 40000, keys, "faces");
			else if (m_SelectLevel == EDGE_TYPE)
				count = HA_Compute_Geometry_Keys(entity, 40000, keys, "edges");
		}
#endif

		assert(count < 40000);

		if (count)
		{
			for (c = 0; c < count; c++)
			{
				HSelectionSet::Select(keys[c], segpath, 0, 0);  // call base class fcn for each key
			}
		}
		else
			HSelectionSet::Select(key, segpath, include_key, includer_key);


}


void HSSelectionSet::DeSelect(long key)
{
	HC_KEY keys[40000];
	int count = 0, c;

#ifdef PARASOLID
	PK_ENTITY_t	entity = 0;
	int			n = 0;

	entity = HP_Compute_TagID(key, m_SelectLevel);

	if (entity > 0)
	{
		while (m_pSolidSelection->At(n) != (long)entity)
		{
			n++;
		}
		m_pSolidSelection->Erase(n);

		if (m_SelectLevel == PK_CLASS_body)  // && 'body segments is true 
			count = HP_Compute_Geometry_Keys(entity, 1, keys, "bodies");
		else if (m_SelectLevel == PK_CLASS_face)
			count = HP_Compute_Geometry_Keys(entity, 40000, keys, "faces");
		else if (m_SelectLevel == PK_CLASS_edge)
			count = HP_Compute_Geometry_Keys(entity, 40000, keys, "edges");
	}

#endif
#ifdef ACIS
	ENTITY* entity = 0;
	int		n = 0;

	entity = HA_Compute_Entity_Pointer(key, m_SelectLevel);

	if (entity)
	{
		while (m_pSolidSelection->At(n) != (long)entity)
		{
			n++;
		}
		m_pSolidSelection->Erase(n);

		if (m_SelectLevel == BODY_TYPE)  // && 'body segments is true 
			count = HA_Compute_Geometry_Keys(entity, 1, keys, "bodies");
		else if (m_SelectLevel == FACE_TYPE)
			count = HA_Compute_Geometry_Keys(entity, 40000, keys, "faces");
		else if (m_SelectLevel == EDGE_TYPE)
			count = HA_Compute_Geometry_Keys(entity, 40000, keys, "edges");
	}
#endif

	assert(count < 40000);

	if (count)
	{
		for (c = 0; c < count; c++)
		{
			HSelectionSet::DeSelect(keys[c]);	// call base class fcn for each key
		}
	}
	else
		HSelectionSet::DeSelect(key);

}


// deselect all items as in DeSelect
void HSSelectionSet::DeSelectAll()
{
	m_pSolidSelection->Reset();
	HSelectionSet::DeSelectAll();
}


void HSSelectionSet::Reset()
{
	m_pSolidSelection->Reset();
	HSelectionSet::Reset();
}


bool HSSelectionSet::IsSelected(long key)
{
	
#ifdef PARASOLID
	if (m_SelectLevel == PK_CLASS_body)  // && body segments is true
	{
		PK_ENTITY_t entity = 0;
		int count;
		
		entity = HP_Compute_TagID(key, m_SelectLevel);

		if (entity > 0)
		{
			count = HP_Compute_Geometry_Keys(entity, 1, &key, "bodies");
			assert(count == 1);
		}
	}
#endif
#ifdef ACIS
	if (m_SelectLevel == BODY_TYPE)
	{
		ENTITY* entity = 0;
		int count;
		
		entity = HA_Compute_Entity_Pointer(key, m_SelectLevel);

		if (entity)
		{
			count = HA_Compute_Geometry_Keys(entity, 1, &key, "bodies");
			assert(count == 1);
		}
	}
#endif

    // call base class function with the computed key
	return HSelectionSet::IsSelected(key);
}


	
